# PIACERE framework
PIACERE aims to increase the productivity of DevOps teams in the development and operation of IaC through the provisioning of an integrated DevSecOps framework. 
DevOps teams can program IaC as if they were programming any software application.

**Table of Contents**
  - [Description](#description)
  - [Repository structure](#repository-structure)
  - [Components](#components)
  - [Contact](#contact)
  - [Acknowledgement](#acknowledgement)

## Description
PIACERE enables the automation of several deployment, configuration and management tasks that otherwise would have to be performed manually by an operator.
PIACERE solution consists of an integrated DevSecOps framework to develop, verify, release, configure, provision, and monitor infrastructure as code. 
The main characteristics of this solution are: 

![VISION](PIACERE_Context_IaC.png) {width="500"}

PIACERE supports the different DevSecOps activities. Using a single **integrated environment to develop (IDE)** infrastructural code will unify the automation of the main DevSecOps activities and will shorten the learning curve for new DevSecOps teams.

PIACERE allows DevSecOps teams to model different infrastructure environments, by means of abstractions, through a novel **DevOps Modelling Language (DOML)**, thus hiding the specificities and technicalities of the current solutions. 
PIACERE also provides an extensible **Infrastructural Code Generator (ICG)**, translating DOML into source files for different existing IaC tools, to reduce the time needed for creating infrastructural code for complex applications. 
The provided **extensibility mechanisms (DOML-E)** ensures the sustainability and longevity of the PIACERE approach and tool-suite (new languages and protocols that can appear in the near future). 

Another key innovation of PIACERE is a comprehensive toolkit for verification and trustworthiness. Firstly, a **verification tool (VT)**, that applies static analysis to both the abstract model and the related infrastructural code, to execute consistency checks and other quality verifications according to identified best practices. 
Secondly, an **IaC Code Security Inspector** that offers a form of Static Analysis Security Testing (SAST) by checking the IaC code against the known cybersecurity issues (misconfigurations, use of non-secure libraries, non-secure configuration patterns). 
Thirdly, a **Component Security Inspector** that by analysing also the IaC code, reports the potential vulnerabilities and proposes potential fixes. 
Fourthly, a **Canary environment** that allows unit testing of the behaviour of the infrastructural code on an isolated environment, which would enable the simulation of conditions for the production environment and identify some of the most common anti-patterns. 

In the Ops part of the DevSecOps lifecycle, PIACERE also presents several key innovations: The **Optimized Platform (IOP)** presents the DevSecOps teams with the most appropriate deployment configurations that best meet their defined constraints out of their catalogue of services, resources and infrastructural elements by means of optimization algorithms. The **Execution Platform** will automatically plan, prepare, and provision the infrastructure and plan, prepare, and install the corresponding software elements needed for the application to seamlessly run. At runtime, PIACERE continuously monitors the metrics associated with the defined measurable NFRs (e.g. performance, availability, and security through the **Runtime Security Monitoring**) and be able to **self-learn**, implementing machine-learning algorithms, and realizing an incremental learning strategy by continuously analysing divergences in the decision boundaries and detecting anomalies in the metrics being collected while retaining only the most up to date data to avoid model degradation. Whenever these self-learning mechanisms detect an anomaly or a potential SLA violation, an alarm is triggered, and a self-healing mechanism launched. A **self-healing** mechanism will entail to launch again an optimization algorithm for the actual problem domain and an automatic execution platform, monitoring and so on.

## Repository structure
The Public repository of PIACERE project is organized as follows:
- [/agents]: contain the monitoring agents that need to be deployed along with the IaC to monitor the infrastructure
- [/demos]: different demos that can serve as a basic examples of using PIACERE
- **[/The Platform]**: is divided in many repositories where the several [components](#Components) of the PIACERE framework can be found

Three branches **Y1**, **Y2**, **Y3** gather the versions delivered in the three main milestones of the project (at M12, M24 and M30). The **main** branch stores the final version of the components, as some development and code fixes could continue until the end of the 36-months long project.

## Components
Diagram of the PIACERE framework:
![FRAMEWORK](https://piacere-project.eu/wp-content/uploads/2023/05/solution.png)
 

| Component                             | Repository                                    |
| --------------------------------------|-----------------------------------------------|
| CSE - Canary Sandbox Environment      | [The Platform/cse]                            |
| IOP - IaC Optimized Platform          | [The Platform/iop]                            |
| Runtime Monitoring                    | [The Platform/runtime-monitoring]             |
| Runtime security monitoring           | [The Platform/runtime-security-monitoring]    |
| DOML - DevSecOps Modelling Language   | [The Platform/doml]                           |
| DOML Model Checker                    | [The Platform/doml-model-checker]             |
| IaC Scan Runner                       | [The Platform/iac-scan-runner]                |
| ICG - Infrastucture as Code Generator | [The Platform/icg]                            |
| IDE                                   | [The Platform/ide]                            |
| IEM - IaC Executor Manager            | [The Platform/iem]                            |
| Self-Healing                          | [The Platform/self-healing]                   |
| Self-Learning                         | [The Platform/self-learning]                  |

## Contact
**Juncal Alonso**, PIACERE project manager, TECNALIA 
[Juncal.Alonso@Tecnalia.com]

## Acknowledgement
This project has received funding from the European Union’s Horizon 2020 research and innovation programme under Grant Agreement No. 101000162. ![EU FLAG](https://icons.iconarchive.com/icons/custom-icon-design/all-country-flag/48/European-Union-Flag-icon.png)

# 
# 
### (TO BE DELETED) *** Uniform approach for component repos  *** (TO BE DELETED)


- README: Include a README file with the following sections (see aprox. model in [iac-scan-runner/README.md](https://git.code.tecnalia.com/piacere/public/the-platform/iac-scan-runner/-/blob/main/README.md), Thx @XLAB! :-))
    - Description of the component
    - Installation
    - Documentation (point to)
    - License 
    - Contact 
    - Acknowledgement
- BRANCHES: Branches in each repository: 
    - **y1**, **y2** and **y3** (M30-jun code), with the respective code
    - **main** (default branch, versions from now on)
        1. If not exist, create **main** in your repo (_Code>Branches>New branch_)
        2. Make it the "default" branch (_Settings>Repository>Branch defaults_)






[/demos]: https://git.code.tecnalia.com/piacere/public/demos
[/agents]: https://git.code.tecnalia.com/piacere/public/agents
[/The Platform]: https://git.code.tecnalia.com/piacere/public/the-platform
[The Platform/cse]: https://git.code.tecnalia.com/piacere/public/the-platform/cse
[The Platform/iop]: https://git.code.tecnalia.com/piacere/public/the-platform/iop
[The Platform/runtime-monitoring]: https://git.code.tecnalia.com/piacere/public/the-platform/runtime-monitoring
[The Platform/runtime-security-monitoring]: https://git.code.tecnalia.com/piacere/public/the-platform/runtime-security-monitoring
[The Platform/doml]: https://git.code.tecnalia.com/piacere/public/the-platform/doml
[The Platform/doml-model-checker]: https://git.code.tecnalia.com/piacere/public/the-platform/doml-model-checker
[The Platform/iac-scan-runner]: https://git.code.tecnalia.com/piacere/public/the-platform/iac-scan-runner
[The Platform/icg]: https://git.code.tecnalia.com/piacere/public/the-platform/icg
[The Platform/ide]: https://git.code.tecnalia.com/piacere/public/the-platform/ide
[The Platform/iem]: https://git.code.tecnalia.com/piacere/public/the-platform/iem
[The Platform/self-healing]: https://git.code.tecnalia.com/piacere/public/the-platform/self-healing
[The Platform/self-learning]: https://git.code.tecnalia.com/piacere/public/the-platform/self-learning

[Juncal.Alonso@Tecnalia.com]: mailto:Juncal.Alonso@Tecnalia.com
